ArenaHistory.ArenaMatchUpdater = {}
local ArenaMatchUpdater = ArenaHistory.ArenaMatchUpdater
local ArenaMatch = ArenaHistory.ArenaMatch
local Logger = ArenaHistory.Logger

ArenaMatchUpdater.logger = Logger.new("ArenaMatchUpdater")
ArenaMatchUpdater.supportsBG = false

ArenaMatchUpdater.TIMER_UPDATE_RATE_S = 10

LibStub:GetLibrary("AceEvent-3.0"):Embed(ArenaMatchUpdater)
LibStub:GetLibrary("AceHook-3.0"):Embed(ArenaMatchUpdater)
LibStub:GetLibrary("AceTimer-3.0"):Embed(ArenaMatchUpdater)

local LibRealmInfo = LibStub:GetLibrary("LibRealmInfo")

function ArenaMatchUpdater:OnEnable(version)
  self.region = LibRealmInfo:GetCurrentRegion()
  self.version = version
  self.logger:debug("Init. Version: %s, Region: %s",
      self.version or "?", self.region or "?")

  self.currentMatch = nil

  self:RegisterEvent("PLAYER_ENTERING_WORLD")
  self:RegisterEvent("ARENA_PREP_OPPONENT_SPECIALIZATIONS")
  self:RegisterEvent("ARENA_OPPONENT_UPDATE")
  self:RegisterEvent("UNIT_NAME_UPDATE")
  self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
  self:RegisterEvent("UNIT_AURA")
  self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
  self:HookScript(WorldStateScoreFrame, "OnShow",
      "WORLD_STATE_SCORE_FRAME_ON_SHOW")

  if self:isInMatch() then
    self:onArenaStart()
  end

  self.timer = self:ScheduleRepeatingTimer("onTimer", self.TIMER_UPDATE_RATE_S)
end

function ArenaMatchUpdater:OnDisable()
  self:UnregisterAllEvents()
  self:UnhookAll()
  self:CancelAllTimers()
end

-- Event Handlers

function ArenaMatchUpdater:PLAYER_ENTERING_WORLD()
  ArenaMatchUpdater.logger:debug("PLAYER_ENTERING_WORLD")
  if not self:isInMatch() then
    self:onArenaEnd()
  else
    self:onArenaStart()
  end
end

function ArenaMatchUpdater:ZONE_CHANGED_NEW_AREA()
  ArenaMatchUpdater.logger:debug("ZONE_CHANGED_NEW_AREA")
  if self:isInMatch() then
    self:onArenaStart()
    self.currentMatch:updateZone()
  end
end

function ArenaMatchUpdater:ARENA_PREP_OPPONENT_SPECIALIZATIONS()
  ArenaMatchUpdater.logger:debug("ARENA_PREP_OPPONENT_SPECIALIZATIONS")
  if self:isInMatch() then
    self:onArenaStart()
    self.currentMatch:updatePlayers()
  end
end

function ArenaMatchUpdater:WORLD_STATE_SCORE_FRAME_ON_SHOW()
  ArenaMatchUpdater.logger:debug("WORLD_STATE_SCORE_FRAME_ON_SHOW")
  if self:isInMatch() then
    self:onArenaStart()
    self:onArenaEnd()
  end
end

function ArenaMatchUpdater:ARENA_OPPONENT_UPDATE(event, unitId, unitType)
  if unitType == "seen" and self:isInMatch() and self.currentMatch ~= nil then
    ArenaMatchUpdater.logger:debug("ARENA_OPPONENT_UPDATE")
    self.currentMatch:updatePlayer(self.currentMatch:getPlayerByUnitId(unitId))
  end
end

function ArenaMatchUpdater:UNIT_NAME_UPDATE(event, unitId)
  if self:isInMatch() and self.currentMatch ~= nil then
    ArenaMatchUpdater.logger:debug("UNIT_NAME_UPDATE, unitId: %s", unitId)
    self.currentMatch:updatePlayer(self.currentMatch:getPlayerByUnitId(unitId))
  end
end

function ArenaMatchUpdater:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp,
    eventType, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags,
    destGUID, destName, destFlags, destRaidFlags, spellID, spellName,
    spellSchool, auraType)
  if self.currentMatch ~= nil and self:isInMatch() then
    if eventType == "UNIT_DIED" then
      self:UNIT_DIED(destGUID)
    end
  end
end

function ArenaMatchUpdater:UNIT_DIED(guid)
  local player = self.currentMatch:getPlayerByGUID(guid)
  if player ~= nil then
    self.logger:info("Player %s died", player.id)
    player:died()
  end
end

function ArenaMatchUpdater:UNIT_AURA(event, unitId)
  if self:isInMatch() and self.currentMatch ~= nil then
    ArenaMatchUpdater.logger:debug("UNIT_AURA, unitId: %s", unitId)
    local player = self.currentMatch:getPlayerByUnitId(unitId)
    if player ~= nil then
      self.currentMatch:updatePlayer(player)
      player:updateAuras()
    end
  end
end

-- Methods

function ArenaMatchUpdater:isInMatch()
  local _, zoneType = IsInInstance()
  return zoneType == "arena" or (self.supportsBG and zoneType == "pvp") 
end

function ArenaMatchUpdater:onTimer()
  if self:isInMatch() and self.currentMatch ~= nil then
    self.currentMatch:updatePlayers()
  end
end

function ArenaMatchUpdater:onArenaStart()
  if self.currentMatch == nil then
    self.logger:debug("onArenaStart")
    self.currentMatch = ArenaMatch.new(REGION)
    self.currentMatch.addonVersion = self.version
    self.currentMatch.region = self.region
  end
end

function ArenaMatchUpdater:onArenaEnd()
  if self.currentMatch ~= nil then
    self.logger:debug("onArenaEnd")
    self.currentMatch:finish()
    self:dumpMatch()
    self.currentMatch = nil
  end
end

function ArenaMatchUpdater:dumpMatch()
  self.logger:info("Arena ended. Result: %d", self.currentMatch.score or 255)
  local dump = self.currentMatch:dump()
  if dump ~= nil then
    self:SendMessage("ARENA_HISTORY_ARENA_MATCH_RECORDED", dump)
  end
end
